/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WIIUSCROLLPOINT_H_
#define WIIUSCROLLPOINT_H_

#include <IntSize.h>
#include <IntPoint.h>
#include <IntSize.h>
#include <WebKit/wiiu/Api/wk_event.h>

namespace WebCore {
	class IntPoint;
	class IntSize;
}

class VelocityCalculator {
public:
	VelocityCalculator(int bufferSize);
	~VelocityCalculator();
	void PointSeen(int x, int y, long long time);
	float XVelocity();
	float YVelocity();
	float VelocitySquared();
	void ClearHistory();

private:
	struct Point {
		int x;
		int y;
		long long time;
	};

	void UpdateVelocity();
	typedef Point* HistoryBuffer;
	HistoryBuffer buffer;

	int index;
	size_t num_valid_entries;
	const size_t buffer_size;
	float x_velocity;
	float y_velocity;
	bool velocities_stale;
};

class WiiUScrollPoint {
public:
	WiiUScrollPoint();
	~WiiUScrollPoint();

	void Reset();
	void ResetVelocity();
	WebCore::IntSize ScrollDelta();
	void UpdateValues(const WiiUEventType_t type, const WiiUMouseEvent& ev);
	void UpdateForScroll();
	bool IsInFlickWindow();
	bool DidScroll(const WiiUMouseEvent& ev, int dist);
	bool IsInHorizontalRailWindow();
	bool IsInVerticalRailWindow();
	bool BreaksHorizontalRail();
	bool BreaksVerticalRail();

	WebCore::IntPoint first_touch_position() {return m_firstTouchPosition;}

	double last_touch_time() {return m_lastTouchTime;}
	WebCore::IntPoint last_touch_position() {return m_lastTouchPosition;}
	int x() {return m_lastTouchPosition.x();}
	int y() {return m_lastTouchPosition.y();}
	float XVelocity() {return velocity_calculator.XVelocity();}
	float YVelocity() {return velocity_calculator.YVelocity();}

private:

	bool IsConsistentScrollingActionUnderway();
	bool IsOverMinFlickSpeed();

	int ScrollVelocityDirection(float v);

	WebCore::IntPoint m_firstTouchPosition;
	long long m_firstTouchTime;

	WebCore::IntPoint m_secondLastTouchPosition;
	long long m_secondLastTouchTime;

	WebCore::IntPoint m_lastTouchPosition;
	long long m_lastTouchTime;

	VelocityCalculator velocity_calculator;

	WebCore::IntSize same_direction_count;
};

#endif /* WIIUSCROLLPOINT_H_ */
